#include "os.h"
#include "kheap.h"
#include "task.h"
#include "initrd.h"
#include "syscall.h"
#include "shared_pages.h"

extern page_directory_t* kernel_directory;
extern page_directory_t* current_directory;
extern ULONG file_data_start;
extern ULONG file_data_end;
ULONG address_user;
unsigned char address_TEST[4096];
unsigned char buf[4096];
unsigned char flag1 = 0; // status of user-space-program
Mem_Chunk_t Mem_Chunk[10]; // contiguous parts of memory detected by int 15h eax = 820h

extern ULONG placement_address;
heap_t* kheap = 0;

static void init()
{
    kheap = 0;
    kernel_directory  = 0;
    current_directory = 0;
    placement_address = 0x200000;
}

int main()
{
    init();
    k_clear_screen(); settextcolor(14,0);
    printformat("PrettyOS [Version 0.1.0091]  (C) 2009 henkessoft.de\n");
    gdt_install();
    idt_install();
    isrs_install();
    irq_install();
    initODA();
    timer_install();
    keyboard_install();

    // get physical memory which is usable RAM
    USHORT num_of_entries = *( (USHORT*)(0x7DFC) );
    pODA->Memory_Size = 0;
    ULONG i,j;
    for(i=0; i<num_of_entries; ++i)
    {
        for(j=0; j<24; j+=4)
        {
            if(j== 0) Mem_Chunk[i].base_lo   = *( (ULONG*)(0x8000+i*24+j) );
            if(j== 4) Mem_Chunk[i].base_hi   = *( (ULONG*)(0x8000+i*24+j) );
            if(j== 8) Mem_Chunk[i].length_lo = *( (ULONG*)(0x8000+i*24+j) );
            if(j==12) Mem_Chunk[i].length_hi = *( (ULONG*)(0x8000+i*24+j) );
            if(j==16) Mem_Chunk[i].type      = *( (ULONG*)(0x8000+i*24+j) );
            if(j==20) Mem_Chunk[i].extended  = *( (ULONG*)(0x8000+i*24+j) );
        }
        if((Mem_Chunk[i].type)==1) pODA->Memory_Size += Mem_Chunk[i].length_lo;
    }
    if( (pODA->Memory_Size>0)&&(pODA->Memory_Size<=0xFFFFFFFF) )
    {
        printformat("\nUsable RAM: %d KB", (pODA->Memory_Size)/1024);
    }
    else
    {
        if(pODA->Memory_Size==0)
           pODA->Memory_Size = 0x2000000; // 32 MB

        printformat("\nMemory detection does not work. Estimated usable RAM: %d KB", (pODA->Memory_Size)/1024);
    }
    printformat("\n\n");

    // paging, kernel heap, multitasking
    paging_install();
    kheap = create_heap(KHEAP_START, KHEAP_START+KHEAP_INITIAL_SIZE, KHEAP_MAX, 1, 0); // SV and RW
    tasking_install(); // ends with sti()

    // shell in elf-executable-format provided by data.asm
    k_memcpy((void*)address_TEST, &file_data_start, (ULONG)&file_data_end - (ULONG)&file_data_start);
    ULONG elf_vaddr     = *( (ULONG*)( address_TEST + 0x3C ) );
    ULONG elf_offset    = *( (ULONG*)( address_TEST + 0x38 ) );
    ULONG elf_filesize  = *( (ULONG*)( address_TEST + 0x44 ) );

    ///
    #ifdef _DIAGNOSIS_
    settextcolor(5,0);
    printformat("virtual address: %x offset: %x size: %x", elf_vaddr, elf_offset, elf_filesize);
    printformat("\n\n");
    settextcolor(15,0);
    #endif
    ///

    if( (elf_vaddr == 0x400000) || (elf_offset<0x130) || (elf_filesize<0x1000) )
    {
        flag1=1;
    }

    // address_TEST (ring 0) ==> address_user (ring 3)
    if(flag1==1)
    {
        address_user = elf_vaddr;
        k_memcpy((void*)address_user, address_TEST + elf_offset, elf_filesize); // ELF LOAD: Offset VADDR FileSize
    }

    pODA->ts_flag = 1; // enable task_switching

    if(flag1==1)
    {
        create_task ((void*)getAddressUserProgram(),3); // program in user space (ring 3) takes over
    }
    else
    {
        settextcolor(4,0);
        printformat("Program not found.\n");
        settextcolor(15,0);

    }

    while(TRUE){/*  */}
    return 0;
}
